/* COPYRIGHT (C) GUPTA TECHNOLOGIES, INC. 1984-1989 */
/*
FUNCTION NAME
  RESSET - RESULT SET API CALLS
REVISION HISTORY
  03/26/90 CHY original
EXTERNAL INTERFACE
  FUNCTION
    Main program for testing api calls pertaining to result sets.
DESCRIPTION
    This utility was to test out the api calls for result set.
RETURN
INTERNAL DESCRIPTION
*/
#include "sql.h"
#include "stdio.h"
#include "string.h"
#include "sqlsrv.h"
#include "errsql.h"

/* error defines */
#define APIERROR    1

#define apierr(a) error(a,__LINE__, APIERROR)

static	  SQLTCUR   cur;
static	  SQLTRCD   rcd;
static	  char*     dbname;
static    char*     curname;

	  main()
{
	  char*     p;			/* misc. pointer		    */
	  FILE*     in; 		/* input file			    */

	  SQLTDAP   cp; 		/* character pointer		    */
	  SQLTDAL   length;		/* length			    */
	  int	    rows;		/* number of rows		    */
	  SQLTPDT   pdt;		/* program data type		    */
	  SQLTBNN   bnn;		/* bind number			    */
	  SQLTNBV   nbv;		/* number of bind variables	    */
	  char	    line[200];		/* I/O line			    */
	  int	    prs=0;
	    
   static char  cname[] = "FRED";	
   static char	defdb[] = "omed";	/* default database name 	    */
   static char  ctbcom[] =		/* CREATE TABLE command 	    */
   "CREATE TABLE X (A NUMBER, B DATETIME, C CHAR(30), D NUMBER)";
   static char  inscom[] =		/* INSERT command		    */
   "INSERT INTO X(A, B, C, D) VALUES (:1, :2, :3, :4)";
   static char  ctincom[] =		/* CREATE INDEX command 	    */
   "CREATE INDEX A ON X(A)";
   static char  selcom1[] =		/* SELECT command		    */
   "SELECT A, D, C FROM X WHERE A < 1000";
   static char  selcom2[] =		/* SELECT command		    */
   "SELECT A, D, C FROM X WHERE A > 1000";  
   static char  selcom3[] =		/* SELECT command		    */
   "SELECT A, D, C FROM X WHERE A < 900";
   static char  selcom4[] =		/* SELECT command		    */
   "SELECT A, D, C FROM X";

   curname = cname;   
   dbname = defdb;
   
  if (rcd = sqlcnc(&cur,defdb,0))
    apierr("SQLCNC");
  
  /*  CREATE THE TABLE  */

  if (sqlcex(cur, ctbcom, 0))		/* failure on compile and execute?  */
    apierr("SQLCEX");

  /*  COMPILE THE INSERT COMMAND  */

  if (sqlcom(cur, inscom, 0))		/* failure on compile?		    */
    apierr("SQLCOM");

  /* INSERT THE DATA */

  if (!(in = fopen("DATA", "r")))	/* failure on open of file?	    */
  {
    printf("tst: can't open data file\n");
    exit(1);
  }

  rows = 0;				/* initialize number of rows	    */
  pdt = SQLPBUF;			/* set program data type of buffer  */

  while (fgets(line, sizeof(line), in)) /* scan the input		    */
  {
    rows++;				/* increment number of rows	    */
    line[strlen(line) - 1] = 0; 	/* remove new line		    */

    if (sqlnbv(cur, &nbv))		/* failure on nbv?		    */
      apierr("SQLNBV");

    for (cp = line, bnn = 1;		/* scan the line		    */
	 bnn <= nbv; bnn++)
    {
      length = (p = strchr(cp, ',')) ?	/* locate comma 		    */
	  p - cp : strlen(cp);
      sqlbnn(cur, bnn, cp, length, 0,	/* bind by number		    */
	  pdt);
      cp += length;			/* locate end			    */
      if (*cp == ',')			/* comma?			    */
	cp++;
    }

    if (sqlexe(cur))			/* failure on execute?		    */
    {
      printf("%d ROWS INSERTED\n", rows);
      apierr("SQLEXE");
    }
  }

  /* COMMIT */

  if (rcd = sqlcmt(cur))
    apierr("SQLCMT");
	  
  /* CREATE THE INDEX */

  if (sqlcex(cur, ctincom, 0))		/* failure on create index?	    */
    apierr("SQLCEX");

  /* SET CURSOR NAME  */

  if (rcd = sqlscn(cur,cname,0))
    apierr("SQLSCN");
	  
  /* START RESULT SET MODE */

  if (rcd = sqlsrs(cur))
    apierr("SQLSRS");

  /* STOP RESTRICTION MODE */
  
  if (rcd = sqlspr(cur))
    apierr("SQLSPR");

  select(selcom1,"first",prs);
  select(selcom2,"second",prs);
  
  if (rcd = sqlcrs(cur,"rel1",0))
    apierr("SQLCRS");

  if (rcd = sqldrs(cur, "rel1",0))
    apierr("SQLDRS");

  /* START RESULT SET MODE */

  if (rcd = sqlsrs(cur))
    apierr("SQLSRS");

  /* START RESTRICTION MODE */
  
  if (rcd = sqlstr(cur))
    apierr("SQLSTR");

  select(selcom1,"first",prs);
  select(selcom3,"third",prs);

  if (rcd = sqlurs(cur))
    apierr("SQLURS");

  select(selcom1,"first",prs);
  
  if (rcd = sqlcrs(cur,"rel2",0))
    apierr("SQLCRS");
	  
  if (rcd = sqlrrs(cur,"rel2",0))
    apierr("SQLRRS");

  prs=1;
  select(selcom4,"fourth",prs);
    
  if (rcd = sqlcmt(cur))
    apierr("SQLCMT");
  if (rcd = sqldis(cur))
    apierr("SQLDIS");

}

static	  select(stmt, sel, prs)
	  char*  stmt;
	  char*  sel;
	  int	 prs;  
  
{
	  SQLTDAP   cp; 		/* character pointer		    */
	  SQLTDAL   length;		/* length			    */
	  int	    rows;		/* number of rows		    */
	  SQLTPDT   pdt;		/* program data type		    */
	  SQLTBNN   bnn;		/* bind number			    */
	  SQLTNBV   nbv;		/* number of bind variables	    */
	  char	    line[200];		/* I/O line			    */

	  SQLTDDT   ddt;		/* database data type		    */
	  SQLTSLC   slc;		/* select list column		    */
	  SQLTNSI   nsi;		/* number of select items	    */
  	  SQLTPDL   pdl;		/* program buffer length	    */
	  
	  SQLTROW   nrr;
	  SQLTROW   prow;
	  
  pdt = SQLPBUF;
  prow = 2;
  
  /* COMPILE THE SELECT COMMAND */
	  
  if (rcd = sqlcom(cur, stmt, 0))	/* failure on select compile?	    */
    apierr("SQLCOM");

  /* GET DESCRIPTIVE INFORMATION ABOUT SELECT */

  cp = line;				/* set pointer to input line	    */

  if (rcd = sqlnsi(cur, &nsi))		/* get # select items		    */
    apierr("SQLNSI");

  for (slc = 1; slc <= nsi; slc++)	/* get information on each column   */
  {
    if (rcd = sqldes(cur, slc, &ddt, &pdl, /* failure on describe? 	    */
	SQLNPTR, SQLNPTR, SQLNPTR, SQLNPTR))
      apierr("SQLDES");
    if (rcd = sqlssb(cur, slc, pdt, cp,	/* set select buffer		    */
	pdl, 0, SQLNPTR, SQLNPTR))
      apierr("SQLSSB");
    cp += (pdl + 1);			/* locate next area		    */
  }

  /* EXECUTE THE SELECT COMMAND */

  if (rcd = sqlexe(cur))		/* failure on select execute?	    */
    apierr("SQLEXE");

  if (prs == 1)
  {
    if (rcd = sqlprs(cur,prow))
      apierr("SQLPRS");
  }
  /* FETCH AND DISPLAY THE DATA */

  length = cp - line;			/* compute the length		    */
  *cp = 0;				/* concatenate a zero to the string */

  for (;;)
  {
    memset(line, ' ', length);		/* fill the line with spaces	    */
    if (rcd = sqlfet(cur))		/* failure or end of file?	    */
      break;
    printf("%s\n", line);		/* print the line		    */
  }

  if (rcd != 1) 			/* failure on fetch		    */
    apierr("SQLFET");

  printf("ROWS FETCHED for %s select\n",sel);

  if (prs==1)
    printf("ROWS FETCHED after positioning to second row in result set \n");
	
  if (rcd = sqlnrr(cur,&nrr))
    apierr("SQLNRR");
  else 
    printf("NUMBER OF ROWS IN RESULT SET for %s SELECT = %d \n",sel, nrr);
	
}

/*
  ERROR - prints an error message for the utility
*/
static	  error(what, line, type)
	  char*     what;		/* description of what happened     */
	  int	    line;		/* line number where error occured  */
	  int	    type;		/* type of error		    */
  {
	  char	    msg[200];		/* error message buffer 	    */

      sqlerr(rcd, msg); 		  /* get the error message	    */
      printf("\nError: %s at line %d: %s\n", what, line, msg);
  }

